import os
import sys

import django
import pytest
from django.conf import settings


@pytest.fixture()
def vocabulary_factory():
    from thesaurus.models import Vocabulary

    def make_vocabulary(**args):
        return Vocabulary.objects.create(**args)

    return make_vocabulary


def pytest_configure():
    BASE_DIR = os.path.abspath(os.path.dirname(__file__))  # NOQA

    sys.path.append(os.path.abspath(os.path.join(BASE_DIR, '..')))

    INSTALLED_APPS = [  # NOQA
        'django.contrib.admin',
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.messages',
        'django.contrib.staticfiles',
        'thesaurus',
    ]

    MIDDLEWARE_CLASSES = [  # NOQA
        'django.middleware.security.SecurityMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',
    ]

    settings.configure(
        SECRET_KEY="test-key",
        DEBUG=False,
        TEMPLATE_DEBUG=False,
        ALLOWED_HOSTS=[],
        INSTALLED_APPS=INSTALLED_APPS,
        MIDDLEWARE_CLASSES=MIDDLEWARE_CLASSES,
        ROOT_URLCONF='tests.urls',
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': os.path.join(BASE_DIR, 'test_db.sqlite3'),
            }
        },
        LANGUAGE_CODE='en-us',
        TIME_ZONE='UTC',
        USE_I18N=True,
        USE_L10N=True,
        USE_TZ=True,
        STATIC_URL='/static/',
    )

    django.setup()



from django.contrib import admin
from parler.admin import TranslatableAdmin

from .models import Collection, Concept, Member, Vocabulary


@admin.register(Concept)
class ConceptAdmin(TranslatableAdmin):
    def concept_label(self):
        return self.label

    concept_label.admin_order_field = 'translations__label'

    list_display = ('code', concept_label, 'vocabulary')
    list_filter = ('vocabulary', )
    search_fields = ('code', 'translations__label')
    model = Concept

    def get_queryset(self, request):
        qs = super().get_queryset(request)

        language_code = self.get_queryset_language(request)

        return qs.prefetch_related('translations', 'vocabulary__translations').translated(language_code)


@admin.register(Collection)
class CollectionAdmin(TranslatableAdmin):
    model = Collection


@admin.register(Member)
class MemberAdmin(admin.ModelAdmin):
    def concept_label(self):
        return self.concept.label

    def parent_concept_label(self):
        return self.parent.concept.label if self.parent else None

    def collection_label(self):
        return self.collection.label if self.collection else None

    list_display = (concept_label, parent_concept_label, collection_label,)
    list_filter = ('concept__vocabulary', 'collection')
    raw_id_fields = ('concept', 'parent', )
    search_fields = ('concept__code', 'concept__translations__label')
    model = Member

    def get_queryset(self, request):
        qs = super().get_queryset(request)

        return qs.select_related('concept', 'collection', 'parent', 'parent__concept').prefetch_related(
            'concept__translations', 'collection__translations', 'parent__concept__translations')


@admin.register(Vocabulary)
class VocabularyAdmin(TranslatableAdmin):
    model = Vocabulary



import rdflib
from django.core.management.base import BaseCommand
from rdflib import RDF
from rdflib.namespace import DC, SKOS

from thesaurus.models import Collection, Concept, Member, Vocabulary


def get_or_create_vocabulary(g, scheme_uri):
    try:
        vocabulary = Vocabulary.objects.get(uri=str(scheme_uri))
    except Vocabulary.DoesNotExist:
        scheme_prefix = None

        for (prefix, namespace) in g.namespace_manager.namespaces():
            if namespace == scheme_uri:
                scheme_prefix = prefix

        vocabulary = Vocabulary.objects.create(uri=scheme_uri, prefix=scheme_prefix)
        collection = Collection.objects.create()

        for title_object in g.objects(subject=scheme_uri, predicate=DC.title):
            vocabulary.set_current_language(title_object.language)
            vocabulary.label = title_object.value
            collection.set_current_language(title_object.language)
            collection.label = title_object.value

        collection.save()
        vocabulary.collection = collection
        vocabulary.save()

    return vocabulary


def create_concept(g, subject):
    data = {
        'uri': str(subject),
        'code': str(subject).split('/')[-1],
        'labels': {},
        'parent_uri': None,
        'scheme': None,
    }

    scheme_uri = g.value(subject, SKOS.inScheme)

    # Ignore concepts not having a scheme for now.
    if not scheme_uri:
        return None

    vocabulary = get_or_create_vocabulary(g, scheme_uri)

    try:
        concept = Concept.objects.get(code=data['code'], vocabulary=vocabulary)

        return concept
    except Concept.DoesNotExist:
        pass

    for (ref, literal) in g.preferredLabel(subject):
        data['labels'][literal.language] = literal.value

    data['parent_uri'] = g.value(subject, SKOS.broader)

    parent_member = None
    # Only consider parent concepts that are part of the same scheme for now.
    if data['parent_uri'] and data['parent_uri'].startswith(scheme_uri):
        parent_concept = create_concept(g, data['parent_uri'])

        try:
            parent_member = Member.objects.get(collection=vocabulary.collection, concept=parent_concept)
        except Member.DoesNotExist:
            pass

    concept = Concept(code=data['code'], vocabulary=vocabulary)

    for (lang, label) in data['labels'].items():
        concept.set_current_language(lang)
        concept.label = label

    concept.save()

    if vocabulary.collection:
        Member.objects.create(collection=vocabulary.collection, concept=concept, parent=parent_member)

    return concept


def import_from_file(file):
    g = rdflib.Graph()
    print("Starting parse. This could take minutes if there are a lot of statements.")
    g.parse(file=file)
    print("Parse ended.")

    print("Importing concepts...")
    count = 0
    for subject in g.subjects(predicate=RDF.type, object=SKOS.Concept):
        create_concept(g, subject)

        count += 1
        if count % 100 == 0:
            print(count)


class Command(BaseCommand):
    help = "Import vocabulary from file(s)"

    def add_arguments(self, parser):
        parser.add_argument('skos_files', nargs="+")

    def handle(self, *args, **options):
        for filename in options['skos_files']:
            with open(filename, "r") as f:
                print("Importing {}.".format(filename))
                import_from_file(f)

        from django.db import connection
        print("Queries: {}".format(len(connection.queries)))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-23 13:32
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import parler.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Collection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'abstract': False,
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='CollectionTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('label', models.CharField(blank=True, max_length=255, null=True)),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='thesaurus.Collection')),
            ],
            options={
                'verbose_name': 'collection Translation',
                'db_table': 'thesaurus_collection_translation',
                'db_tablespace': '',
                'managed': True,
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='Concept',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(blank=True, max_length=255, null=True, unique=True)),
            ],
            options={
                'abstract': False,
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ConceptTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('label', models.CharField(blank=True, max_length=255, null=True)),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='thesaurus.Concept')),
            ],
            options={
                'verbose_name': 'concept Translation',
                'db_table': 'thesaurus_concept_translation',
                'db_tablespace': '',
                'managed': True,
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='Member',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='thesaurus.Collection')),
                ('concept', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='thesaurus.Concept')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='thesaurus.Member')),
            ],
        ),
        migrations.CreateModel(
            name='Vocabulary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('prefix', models.CharField(blank=True, max_length=255, null=True)),
                ('uri', models.CharField(blank=True, max_length=255, null=True, unique=True)),
                ('collection', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='thesaurus.Collection')),
            ],
            options={
                'abstract': False,
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='VocabularyTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('label', models.CharField(blank=True, max_length=255, null=True)),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='thesaurus.Vocabulary')),
            ],
            options={
                'verbose_name': 'vocabulary Translation',
                'db_table': 'thesaurus_vocabulary_translation',
                'db_tablespace': '',
                'managed': True,
                'default_permissions': (),
            },
        ),
        migrations.AddField(
            model_name='concept',
            name='collections',
            field=models.ManyToManyField(related_name='concepts', through='thesaurus.Member', to='thesaurus.Collection'),
        ),
        migrations.AddField(
            model_name='concept',
            name='vocabulary',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, to='thesaurus.Vocabulary'),
        ),
        migrations.AlterUniqueTogether(
            name='vocabularytranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='concepttranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='collectiontranslation',
            unique_together=set([('language_code', 'master')]),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-thesaurus',
    version='0.0.1',
    packages=['thesaurus'],
    include_package_data=True,
    license='MIT License',
    description='A Django app for importing and using thesauri.',
    long_description=README,
    url='https://github.com/City-of-Helsinki/django-thesaurus',
    author='Mikko Keskinen',
    author_email='keso@iki.fi',
    setup_requires=[
        'pytest-runner',
    ],
    tests_require=[
        'pytest',
        'pytest-cov',
        'pytest-django',
    ],
    install_requires=[
        'Django',
        'django-parler',
        'rdflib',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from parler.models import TranslatableModel, TranslatedFields


@python_2_unicode_compatible
class Vocabulary(TranslatableModel):
    translations = TranslatedFields(
        label=models.CharField(max_length=255, null=True, blank=True)
    )
    prefix = models.CharField(max_length=255, null=True, blank=True)
    uri = models.CharField(max_length=255, unique=True, null=True, blank=True)
    # The collection consisting of all the concepts in a vocabulary.
    # It's automatically created when a vocabulary is imported.
    collection = models.ForeignKey('Collection', on_delete=models.SET_NULL, null=True, blank=True)

    def __str__(self):
        return "{} ({})".format(self.safe_translation_getter("label", any_language=True), self.prefix)


@python_2_unicode_compatible
class Concept(TranslatableModel):
    translations = TranslatedFields(
        label=models.CharField(max_length=255, null=True, blank=True)
    )
    code = models.CharField(max_length=255, unique=True, null=True, blank=True)
    vocabulary = models.ForeignKey(Vocabulary, on_delete=models.CASCADE, blank=True)

    collections = models.ManyToManyField('Collection', through='Member', related_name='concepts')

    def __str__(self):
        return "{} ({})".format(self.safe_translation_getter("label", any_language=True), self.code)


@python_2_unicode_compatible
class Collection(TranslatableModel):
    translations = TranslatedFields(
        label=models.CharField(max_length=255, null=True, blank=True)
    )

    def __str__(self):
        label = self.safe_translation_getter("label", any_language=True)

        return label if label else '(EMPTY LABEL)'


@python_2_unicode_compatible
class Member(models.Model):
    collection = models.ForeignKey(Collection, on_delete=models.CASCADE)
    concept = models.ForeignKey(Concept, on_delete=models.CASCADE)
    parent = models.ForeignKey('self', null=True, blank=True, related_name='children', db_index=True)



import os

import pytest
import rdflib
from rdflib import RDF
from rdflib.namespace import SKOS

from thesaurus.management.commands.import_skos import create_concept
from thesaurus.models import Concept, Vocabulary


@pytest.mark.django_db
def test_import_rdf(vocabulary_factory):
    g = rdflib.Graph()

    test_rdf_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'fixtures/example.rdf'))

    with open(test_rdf_path, "r") as f:
        g.parse(file=f)

    for subject in g.subjects(predicate=RDF.type, object=SKOS.Concept):
        create_concept(g, subject)

    vocabulary = Vocabulary.objects.get(uri='http://example.com/onto/example/')

    assert bool(vocabulary.collection)
    assert vocabulary.prefix == 'example'

    assert Concept.objects.all().count() == 6


@pytest.mark.django_db
def test_import_ttl(vocabulary_factory):
    g = rdflib.Graph()

    test_rdf_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'fixtures/example.ttl'))

    with open(test_rdf_path, "r") as f:
        g.parse(file=f, format='turtle')

    for subject in g.subjects(predicate=RDF.type, object=SKOS.Concept):
        create_concept(g, subject)

    vocabulary = Vocabulary.objects.get(uri='http://example.com/onto/example/')

    assert bool(vocabulary.collection)
    assert vocabulary.prefix == 'example'

    assert Concept.objects.all().count() == 6
